#ifndef _QLCNIC_COMPAT_H
#define _QLCNIC_COMPAT_H

#include <linux/netdevice.h>
#include <linux/version.h>
#include <linux/log2.h>
#include <linux/if_vlan.h>
#include <linux/ethtool.h>

#ifndef ETH_TEST_FL_EXTERNAL_LB
#define ETH_TEST_FL_EXTERNAL_LB  (1 << 2)
#endif

#ifndef ETH_TEST_FL_EXTERNAL_LB_DONE
#define ETH_TEST_FL_EXTERNAL_LB_DONE  (1 << 3)
#endif

void qlcnic_cancel_work(struct workqueue_struct *, struct work_struct *);

#ifndef SPEED_UNKNOWN
#define SPEED_UNKNOWN 0x0
#endif

#ifndef DUPLEX_UNKNOWN
#define DUPLEX_UNKNOWN 0xff
#endif

#ifndef rounddown_pow_of_two
#define qlcnic_rounddown_pow_of_two
unsigned long __rounddown_pow_of_two(unsigned long);
#define rounddown_pow_of_two(n)                 \
(                                               \
        __builtin_constant_p(n) ? (             \
                (n == 1) ? 0 :                  \
                (1UL << ilog2(n))) :            \
        __rounddown_pow_of_two(n)               \
)
#endif
#if (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(5, 7))
#ifndef NETIF_F_VLAN_SG
#define NETIF_F_VLAN_SG 0
#endif
void * vzalloc(unsigned long size);
#endif
/*******************Rhel5.5/5.4 ********************************/
#if (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(5, 6))
typedef int (*disable_lro_func_t)(struct net_device *);
int register_lro_netdev(struct net_device *,
		disable_lro_func_t);
int unregister_lro_netdev(struct net_device *);

#ifndef PORT_OTHER
#define PORT_OTHER	0xff
#endif

#ifndef pci_ioremap_bar
#define pci_ioremap_bar(PDEV, BAR) \
	ioremap_nocache(pci_resource_start(PDEV, BAR), \
			pci_resource_len(PDEV, BAR));
#endif

#ifndef work_func_t
typedef void (*work_func_t)(void *);
#endif


#ifndef NETIF_F_LRO
#define	NETIF_F_LRO		32768		/* large receive offload */
#endif

#ifndef to_net_dev
#define to_net_dev(class) container_of(class, struct net_device, class_dev)
#endif

#ifndef DEFINE_PCI_DEVICE_TABLE
#define DEFINE_PCI_DEVICE_TABLE(_table) \
         struct pci_device_id _table[] __devinitdata
#endif

#ifndef NETIF_F_VLAN_CSUM
#define NETIF_F_VLAN_CSUM (1 << 30)
#endif

#ifndef NETIF_F_VLAN_TSO
#define NETIF_F_VLAN_TSO (1 << 31)
#endif

inline struct net_device *vlan_group_get_device(struct vlan_group *vg,
					int vlan_id);
static inline const char *netdev_name(const struct net_device *dev)
{
	if (dev->reg_state != NETREG_REGISTERED)
		return "(unregistered net_device)";
	return dev->name;
}

#define netdev_printk(level, netdev, format, args...)		\
	dev_printk(level, (netdev)->class_dev.dev,		\
		   "%s: " format,				\
		   netdev_name(netdev), ##args)
#define netdev_err(dev, format, args...)			\
	netdev_printk(KERN_ERR, dev, format, ##args)
#define netdev_warn(dev, format, args...)			\
	netdev_printk(KERN_WARNING, dev, format, ##args)
#define netdev_notice(dev, format, args...)			\
	netdev_printk(KERN_NOTICE, dev, format, ##args)
#define netdev_info(dev, format, args...)			\
	netdev_printk(KERN_INFO, dev, format, ##args)

#define netif_printk(priv, type, level, dev, fmt, args...)	\
do {					  			\
	if (netif_msg_##type(priv))				\
		netdev_printk(level, (dev), fmt, ##args);	\
} while (0)

#define netif_level(level, priv, type, dev, fmt, args...)	\
do {								\
	if (netif_msg_##type(priv))				\
		netdev_##level(dev, fmt, ##args);		\
} while (0)

#define netif_emerg(priv, type, dev, fmt, args...)		\
	netif_level(emerg, priv, type, dev, fmt, ##args)
#define netif_alert(priv, type, dev, fmt, args...)		\
	netif_level(alert, priv, type, dev, fmt, ##args)
#define netif_crit(priv, type, dev, fmt, args...)		\
	netif_level(crit, priv, type, dev, fmt, ##args)
#define netif_err(priv, type, dev, fmt, args...)		\
	netif_level(err, priv, type, dev, fmt, ##args)
#define netif_warn(priv, type, dev, fmt, args...)		\
	netif_level(warn, priv, type, dev, fmt, ##args)
#define netif_notice(priv, type, dev, fmt, args...)		\
	netif_level(notice, priv, type, dev, fmt, ##args)
#define netif_info(priv, type, dev, fmt, args...)		\
	netif_level(info, priv, type, dev, fmt, ##args)

#endif /****END Rhel5.5/5.4 ******/

#ifndef netdev_tx_t
#define netdev_tx_t	int
#endif

#ifndef list_splice_tail_init
#define list_splice_tail_init list_splice_init
#endif

#ifndef vlan_dev_real_dev
#define vlan_dev_real_dev(dev) (VLAN_DEV_INFO(dev)->real_dev)
#endif

#ifndef netdev_mc_count
#define netdev_mc_count(dev) ((dev)->mc_count)
#endif

#ifndef netdev_mc_empty
#define netdev_mc_empty(dev) (netdev_mc_count(dev) == 0)
#endif

#ifndef netdev_for_each_mc_addr
#define netdev_for_each_mc_addr(mclist, dev) \
	for (mclist = dev->mc_list; mclist; mclist = mclist->next)
#endif

#endif

